@@ -9,6 +9,7 @@
  * January,February,July,September-November 2000
  * January-May,June,October 2001
  * May-August 2002
+ * February 2003
  *
  *
  * LVM driver is free software; you can redistribute it and/or modify
@@ -222,6 +223,10 @@
  *    22/07/2002 - streamlined blk_ioctl() call
  *    14/08/2002 - stored fs handle in lvm_do_lv_rename
  *                 [kaoru@bsd.tnes.nec.co.jp]
+ *    06/02/2003 - fix persistent snapshot extend/reduce bug in
+ *		   lvm_do_lv_extend_reduce() [dalestephenson@mac.com]
+ *    04/03/2003 - snapshot extend/reduce memory leak
+ *               - VG PE counter wrong [dalestephenson@mac.com]
  *
  */
 
@@ -2697,7 +2702,7 @@ static int __extend_reduce(vg_t * vg_ptr, lv_t * old_lv, lv_t * new_lv)
 		}
 	}
 
-	/* save availiable i/o statistic data */
+	/* save available i/o statistic data */
 	if (old_lv->lv_stripes < 2) {	/* linear logical volume */
 		end = min(old_lv->lv_current_le, new_lv->lv_current_le);
 		for (l = 0; l < end; l++) {
@@ -2773,7 +2778,7 @@ static int lvm_do_lv_extend_reduce(int minor, char *lv_name, lv_t * new_lv)
 	if (r)
 		return r;
 
-	/* copy relevent fields */
+	/* copy relevant fields */
 	down_write(&old_lv->lv_lock);
 
 	if (new_lv->lv_access & LV_SNAPSHOT) {
@@ -2782,6 +2787,8 @@ static int lvm_do_lv_extend_reduce(int minor, char *lv_name, lv_t * new_lv)
 		size *= sizeof(lv_block_exception_t);
 		memcpy(new_lv->lv_block_exception,
 		       old_lv->lv_block_exception, size);
+		vfree(old_lv->lv_block_exception);
+		vfree(old_lv->lv_snapshot_hash_table);
 
 		old_lv->lv_remap_end = new_lv->lv_remap_end;
 		old_lv->lv_block_exception = new_lv->lv_block_exception;
@@ -2792,15 +2799,16 @@ static int lvm_do_lv_extend_reduce(int minor, char *lv_name, lv_t * new_lv)
 		old_lv->lv_snapshot_hash_mask =
 		    new_lv->lv_snapshot_hash_mask;
 
-		for (e = 0; e < new_lv->lv_remap_ptr; e++)
+		for (e = 0; e < old_lv->lv_remap_ptr; e++)
 			lvm_hash_link(new_lv->lv_block_exception + e,
 				      new_lv->lv_block_exception[e].
 				      rdev_org,
 				      new_lv->lv_block_exception[e].
 				      rsector_org, new_lv);
 
+		vg_ptr->pe_allocated -= old_lv->lv_allocated_le;
+		vg_ptr->pe_allocated += new_lv->lv_allocated_le;
 	} else {
-
 		vfree(old_lv->lv_current_pe);
 		vfree(old_lv->lv_snapshot_hash_table);
 
