@@ -125,7 +125,7 @@ int main ( int argc, char **argv) {
    char *cmd_fsck = "e2fsck";
    /* FIXME check "resize.<fstype>" and "oresize.<fstype>" instead */
    char *cmd_resize = NULL;
-   char *cmd_resize_opts = NULL;
+   char *cmd_resize_opts = "";
    char *cmd_resize_default = "resize2fs";
    char *cmd_resize_opts_default = "-p";
    char *cmd_resize_default2 = "ext2resize";
