@@ -1,11 +1,13 @@
 /*
  * tools/lib/pv_check_name.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * May,September 1998
- * January,March,April 1999
+ * January,March,April,July 1999
+ * February 2000
+ * February 2002
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -36,53 +38,29 @@
  *    17/03/1999 - added loop device support
  *    22/04/1999 - DAC960 (/dev/rd/) support by Matthew Taylor
  *                 <M.Taylor@rbgkew.org.uk>
+ *    23/07/1999 - added support for Compac Smart Array controller
+ *    31/01/2000 - use debug_enter()/debug_leave()
+ *    07/02/2002 - removed lvm_check_dev() so that non existing devices
+ *                 don't block vgcfgrestore
  *
  */
 
 #include <liblvm.h>
 
-int pv_check_name ( char *pv_name) {
-   int i = 0;
-   int ret = 0;
-   struct stat stat_b;
-   struct {
-      char *directory;
-      int length;
-   } dir[] = {
-      { "/dev/ide/hd/", 12},
-      { "/dev/loop/",   10},
-      { "/dev/md/",      8},
-      { "/dev/rd/",      8},
-      { "/dev/sd/",      8},
-      { "/dev/",         5},	/* must be last one */
-      { NULL,            0}
-   };
+int pv_check_name(char *pv_name)
+{
+	int ret = 0;
 
-#ifdef DEBUG
-   debug ( "pv_check_name -- CALLED\n");
-#endif
+	debug_enter("pv_check_name -- CALLED with \"%s\"\n", pv_name);
 
-   if ( pv_name == NULL) ret = -LVM_EPARAM;
-   else if ( lvm_check_chars ( pv_name) < 0) ret = -LVM_EPARAM;
-   else {
-      while ( dir[i].directory != NULL) {
-         if ( strncmp ( pv_name, dir[i].directory, dir[i].length) == 0) break;
-         i++;
-      }
-      if ( dir[i].directory == NULL) ret = -LVM_EPV_CHECK_NAME;
-   }
+	if (!pv_name)
+		ret = -LVM_EPARAM;
 
-   if ( ret == 0) {
-      if ( stat ( pv_name, &stat_b) == -1) ret = -LVM_EPV_CHECK_NAME_STAT;
-      else if ( lvm_check_dev ( &stat_b, TRUE) == FALSE)
-         ret = -LVM_EPV_CHECK_NAME;
-   }
+	else if (strncmp(pv_name, LVM_DIR_PREFIX, strlen(LVM_DIR_PREFIX)))
+		ret = -LVM_EPV_CHECK_NAME;
 
-#ifdef DEBUG
-   if ( ret == 0) {
-      debug ( "pv_check_name -- CALLED  pv_name: \"%s\"\n", pv_name);
-   }
-   debug ( "pv_check_name -- LEAVING with ret: %d\n", ret);
-#endif
-   return ret;
+	ret = lvm_check_chars(pv_name);
+
+	debug_leave("pv_check_name -- LEAVING with ret: %d\n", ret);
+	return ret;
 }
