@@ -29,10 +29,8 @@ kdev_t pv_get_kdev_t_by_number ( vg_t *vg, int pv_number) {
    int p;
    kdev_t ret = -1;
 
-#ifdef DEBUG
    debug_enter ( "pv_get_kdev_t_by_number -- CALLED for VG \"%s\" and %d\n",
                  vg->vg_name, pv_number);
-#endif
 
    if ( vg != NULL && vg_check_name ( vg->vg_name) == 0) {
       for ( p = 0; p < vg->pv_max; p++) {
@@ -43,8 +41,6 @@ kdev_t pv_get_kdev_t_by_number ( vg_t *vg, int pv_number) {
       }
    } else ret = -LVM_EPARAM;
 
-#ifdef DEBUG
    debug_leave ( "pv_get_kdev_t_by_number -- LEAVING with ret: %d\n", ret);
-#endif
    return ret;
 }
