@@ -493,7 +493,7 @@ pv_move_pe_continue:
    }
 
    if ( opt_v > 0)
-       printf ( "%s -- %s [PE %lu [%s [LE %d]] -> %s [PE %lu] [%d/%d]\n",
+       printf ( "%s -- %s [PE %lu [%s [LE %d]]] -> %s [PE %lu] [%d/%d]\n",
                 cmd,
                 vg->pv[src_pv_index]->pv_name,
                 pe_source,
