@@ -1,10 +1,11 @@
 /*
  * tools/lib/pv_write_with_pe.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * August-September 1998
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -24,25 +25,29 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    08/02/2000 - use debug_enter()/debug_leave()
+ *
+ */
+  
 #include <liblvm.h>
 
 
 int pv_write_with_pe ( char *pv_name, pv_t *pv) {
    int ret = 0;
 
-#ifdef DEBUG
-   debug ( "pv_write_with_pe -- CALLED with pv->pe_total: %lu\n",
-            pv->pe_total);
-#endif
+   debug_enter ( "pv_write_with_pe -- CALLED with pv->pe_total: %lu\n",
+                 pv->pe_total);
 
    if ( pv_name == NULL ||
         pv_check_name ( pv_name) < 0 ||
-        pv == NULL) return -LVM_EPARAM;
+        pv == NULL) ret = -LVM_EPARAM;
+   else if ( ( ret = pv_write ( pv_name, pv)) == 0) {
+      ret = pv_write_pe ( pv_name, pv);
+   }
 
-   if ( ( ret = pv_write ( pv_name, pv)) == 0) ret = pv_write_pe ( pv_name, pv);
-
-#ifdef DEBUG
-   debug ( "pv_write_with_pe -- LEAVING\n");
-#endif
+   debug_leave ( "pv_write_with_pe -- LEAVING with ret: %d\n", ret);
    return ret;
 }
