@@ -1,13 +1,14 @@
 /*
  * tools/vgchange.c
  *
- * Copyright (C) 1997 - 2001  Heinz Mauelshagen, Sistina Software
+ * Copyright (C) 1997 - 2002  Heinz Mauelshagen, Sistina Software
  *
  * March-May,October 1997
  * May,June,September 1998
  * February,July,October,December 1999
  * February,July 2000
  * June 2001
+ * February 2002
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,6 +56,8 @@
  *               - replaced vg->lv[l] by lv_ptr
  *    09/04/2001 - Tidy usage message and command-line options.
  *    22/06/2001 - added Andreas Dilger's PE on 4k boundary alignment enhancements
+ *    12/02/2002 - drop snapshots where we find an error while setting up
+ *                 or loading the COW table
  *
  */
 
@@ -66,6 +69,9 @@ char *cmd = NULL;
 int opt_d = 0;
 #endif
 
+#define	CREATE	1
+#define	REMOVE	2
+
 void usage ( int ret)
 {
     FILE *stream = stderr;
@@ -161,10 +167,10 @@ int main ( int argc, char **argv) {
             }
             opt_a++;
             if ( strcmp ( optarg, "y") == 0) {
-               opt_a_action = VG_CREATE;
+               opt_a_action = CREATE;
                break;
             } else if ( strcmp ( optarg, "n") == 0) {
-               opt_a_action = ~VG_CREATE;
+               opt_a_action = REMOVE;
                break;
             }
             fprintf ( stderr, "%s -- a option argument \"%s\" invalid\n\n",
@@ -295,63 +301,65 @@ int main ( int argc, char **argv) {
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking existence of volume "
-                               "group \"%s\"\n",
-                               cmd, vg_name);
-      if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) != TRUE) {
-         if ( ret == -LVM_EPV_READ_PV_EXPORTED)
-            fprintf ( stderr, "%s -- can't access exported volume "
-                              "group \"%s\"\n",
-                              cmd, vg_name);
-         else if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT)
-            fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
-                              "volume group \"%s\" online\n", cmd, vg_name);
-         else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_NP)
-            fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
-                      cmd, vg_name);
-         else if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION)
-            fprintf ( stderr, "%s -- volume group \"%s\" has physical volumes "
-                              "with invalid version\n",
-                      cmd, vg_name);
-         else
-            fprintf ( stderr, "%s -- volume group \"%s\" does not exist\n",
-                              cmd, vg_name);
-         continue;
-      }
-
-      if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
-                               "from lvmtab\n", cmd, vg_name);
-      if ( vg != NULL) {
-         vg_free ( vg, FALSE);
-         vg = NULL;
-      }
-
-      if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
-         if ( ret == -LVM_EVG_CFGRESTORE_OPEN) {
-            fprintf ( stderr, "%s -- ERROR: volume group \"%s\" "
-                              "doesn't exist\n",
-                              cmd, vg_name);
-         } else {
-            fprintf ( stderr, "%s -- ERROR \"%s\" reading lvmtab data of "
-                              "volume group \"%s\"\n",
-                              cmd, lvm_error ( ret), vg_name);
+      if ( opt_a_action != REMOVE) {
+         if ( opt_v > 0) printf ( "%s -- checking existence of volume "
+                                  "group \"%s\"\n",
+                                  cmd, vg_name);
+         if ( ( ret = lvm_tab_vg_check_exist ( vg_name, NULL)) != TRUE) {
+            if ( ret == -LVM_EPV_READ_PV_EXPORTED)
+               fprintf ( stderr, "%s -- can't access exported volume "
+                                 "group \"%s\"\n",
+                                 cmd, vg_name);
+            else if ( ret == -LVM_EVG_CHECK_EXIST_PV_COUNT)
+               fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
+                                 "volume group \"%s\" online\n", cmd, vg_name);
+            else if ( ret == -LVM_EPV_READ_ALL_PV_OF_VG_NP)
+               fprintf ( stderr, "%s -- volume group \"%s\" doesn't exist\n",
+                         cmd, vg_name);
+            else if ( ret == -LVM_EVG_READ_LVM_STRUCT_VERSION)
+               fprintf ( stderr, "%s -- volume group \"%s\" has physical "
+                                 "volumes with invalid version\n",
+                         cmd, vg_name);
+            else
+               fprintf ( stderr, "%s -- volume group \"%s\" does not exist\n",
+                                 cmd, vg_name);
+            continue;
          }
-         continue;
-      }
    
-      if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
-                               " of \"%s\"\n", cmd, vg_name);
-      if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" "
-                           " is inconsistent\n",
-                           cmd, lvm_error ( ret), vg_name);
-         continue;
+         if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
+                                  "from lvmtab\n", cmd, vg_name);
+         if ( vg != NULL) {
+            vg_free ( vg, FALSE);
+            vg = NULL;
+         }
+   
+         if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) < 0) {
+            if ( ret == -LVM_EVG_CFGRESTORE_OPEN) {
+               fprintf ( stderr, "%s -- ERROR: volume group \"%s\" "
+                                 "doesn't exist\n",
+                                 cmd, vg_name);
+            } else {
+               fprintf ( stderr, "%s -- ERROR \"%s\" reading lvmtab data of "
+                                 "volume group \"%s\"\n",
+                                 cmd, lvm_error ( ret), vg_name);
+            }
+            continue;
+         }
+      
+         if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
+                                  " of \"%s\"\n", cmd, vg_name);
+         if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
+            fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" "
+                              " is inconsistent\n",
+                              cmd, lvm_error ( ret), vg_name);
+            continue;
+         }
       }
 
       lvm_dont_interrupt ( 0);
    
       if ( opt_a > 0) {
-         if ( opt_a_action & VG_CREATE) {
+         if ( opt_a_action == CREATE) {
             if ( opt_v > 0) printf ( "%s -- checking if all physical volumes "
                                      "of volume group \"%s\" are available\n",
                                      cmd, vg_name);
@@ -383,44 +391,57 @@ int main ( int argc, char **argv) {
       
             /* check for snapshot logical volumes and get the COW table data */
             for ( l = 0; l < vg->lv_max; l++) {
-               if ( ( lv_ptr = vg->lv[l]) != NULL) {
-                  lv_ptr->lv_block_exception = NULL;
-                  lv_ptr->lv_remap_ptr = 0;
-                  lv_ptr->lv_remap_end = 0;
-                  lv_ptr->lv_snapshot_prev = NULL;
-                  lv_ptr->lv_snapshot_next = NULL;
+               lv_ptr = vg->lv[l];
+               if ( lv_ptr == NULL) continue;
 
-                  if ( lv_ptr->lv_access & LV_SNAPSHOT_ORG) lv_ptr->lv_snapshot_org = lv_ptr;
+               lv_ptr->lv_block_exception = NULL;
+               lv_ptr->lv_remap_ptr = 0;
+               lv_ptr->lv_remap_end = 0;
+               lv_ptr->lv_snapshot_prev = NULL;
+               lv_ptr->lv_snapshot_next = NULL;
 
-                  if ( lv_ptr->lv_access & LV_SNAPSHOT) {
-                     if ( ( ret = lv_setup_COW_table_for_create ( vg, vg->lv[lv_get_index_by_minor(vg, lv_ptr->lv_snapshot_minor)]->lv_name, l, lv_ptr->lv_chunk_size / 2)) < 0) {
-                        fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot "
-                                          "copy on write exception table for \"%s\"\n\n",
-                                          cmd, lvm_error ( ret), lv_ptr->lv_name);
-                        return LVM_EVGCHANGE_LV_SETUP_COW_TABLE_FOR_CREATE;
-                     }
-                     if ( ( ret = lv_read_COW_table ( vg, lv_ptr)) < 0) {
-                           fprintf ( stderr, "%s -- ERROR %d reading copy on "
-                                             "write exception table of \"%s\"\n",
-                                             cmd, ret, lv_ptr->lv_name);
-                           return LVM_EVGCHANGE_LV_READ_COW_TABLE;
-                     }
-                     if ( lv_ptr->lv_block_exception[0].rsector_org ==
-                          LVM_SNAPSHOT_DROPPED_SECTOR) {
-                        lv_ptr->lv_status &= ~LV_ACTIVE;
-                        /* Needs to be 0 to avoid that the driver tries
-                           to hash link crap */
-                        lv_ptr->lv_remap_ptr = 0;
-                        printf ( "%s -- INFO: logical volume \"%s\" is a "
-                                 "dropped snapshot.\n"
-                                 "%s -- You can't use it any longer,"
-                                 " remove it please\n",
-                                 cmd, lv_ptr->lv_name,cmd);
-                     }
-                  } else {
-                     lv_ptr->lv_snapshot_minor = 0;
-                     lv_ptr->lv_chunk_size = 0;
+               if ( lv_ptr->lv_access & LV_SNAPSHOT_ORG)
+                   lv_ptr->lv_snapshot_org = lv_ptr;
+
+               if ( lv_ptr->lv_access & LV_SNAPSHOT) {
+                  if ( ( ret = lv_setup_COW_table_for_create ( vg, vg->lv[lv_get_index_by_minor(vg, lv_ptr->lv_snapshot_minor)]->lv_name, l, lv_ptr->lv_chunk_size / 2)) < 0) {
+                     fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot "
+                                       "copy on write exception table for "
+                                       "\"%s\"\n%s -- WARNING: dropping "
+                                       "\"%s\"\n",
+                                       cmd, lvm_error ( ret), lv_ptr->lv_name,
+                                       cmd, lv_ptr->lv_name);
+                     free ( lv_ptr->lv_current_pe);
+                     free ( lv_ptr);
+                     vg->lv[l] = NULL;
+                     continue;
                   }
+                  if ( ( ret = lv_read_COW_table ( vg, lv_ptr)) < 0) {
+                     fprintf ( stderr, "%s -- ERROR %d reading copy on "
+                                       "write exception table of \"%s\"\n"
+                                       "%s -- WARNING: dropping \"%s\"\n",
+                                       cmd, ret, lv_ptr->lv_name,
+                                       cmd, lv_ptr->lv_name);
+                     free ( lv_ptr->lv_current_pe);
+                     free ( lv_ptr);
+                     vg->lv[l] = NULL;
+                     continue;
+                  }
+                  if ( lv_ptr->lv_block_exception[0].rsector_org ==
+                       LVM_SNAPSHOT_DROPPED_SECTOR) {
+                     lv_ptr->lv_status &= ~LV_ACTIVE;
+                     /* Needs to be 0 to avoid that the driver tries
+                        to hash link crap */
+                     lv_ptr->lv_remap_ptr = 0;
+                     printf ( "%s -- INFO: logical volume \"%s\" is a "
+                              "dropped snapshot.\n"
+                              "%s -- You can't use it any longer,"
+                              " remove it please\n",
+                              cmd, lv_ptr->lv_name,cmd);
+                  }
+               } else {
+                  lv_ptr->lv_snapshot_minor = 0;
+                  lv_ptr->lv_chunk_size = 0;
                }
             }
 
@@ -466,7 +487,10 @@ int main ( int argc, char **argv) {
             else {
                debug ( "%s -- vg_remove returned: %d\n", cmd, ret);
                if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN) {
-                  fprintf ( stderr, "%s -- volume group \"%s\" is not active\n", cmd, vg_name);
+                  fprintf ( stderr, "%s -- couldn't  open volume group \"%s\" "
+                                    "group special file\n"
+                                    "%s -- try vgmknodes\n",
+                                    cmd, vg_name, cmd);
                } else if ( ret == -EPERM) {
                   if ( opt_v > 0)
                      printf ( "%s -- getting volume group status from "
